package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.InvocationBuilder;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public final class ChartBuilder {

	private IdGenerator idGenerator;
	private InvocationBuilder<MethodInvocation> invocationBuilder;
	
	public ChartBuilder() {
		this.idGenerator = new IdGenerator();
		this.invocationBuilder = new InvocationBuilder<MethodInvocation>();
	}
	
	public ChartBuilder(IdGenerator idGenerator, InvocationBuilder<MethodInvocation> invocationBuilder) {
		this.idGenerator = idGenerator;
		this.invocationBuilder = invocationBuilder;
	}

	public ScatterChart newScatter() {
		ScatterChart g = new ScatterChart(idGenerator.getNext(),"ScatterChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
	
	public TimeSeriesScatterChart newTimeSeriesScatter() {
		TimeSeriesScatterChart g = new TimeSeriesScatterChart(idGenerator.getNext(),"TimeSeriesScatterChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
	
	public BarLineChart newBarLine() {
		BarLineChart g = new BarLineChart(idGenerator.getNext(), "BarLineChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}

	public BubbleChart newBubble() {
		BubbleChart g = new BubbleChart(idGenerator.getNext(), "BubbleChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}	
	
	public PieChart newPie() {
		PieChart g = new PieChart(idGenerator.getNext(), "PieChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}

	public WaterfallChart newWaterfall() {
		WaterfallChart g = new WaterfallChart(idGenerator.getNext(), "WaterfallChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
	
	public WaterfallComparisonChart newWaterfallComparison() {
		WaterfallComparisonChart g = new WaterfallComparisonChart(idGenerator.getNext(), "WaterfallComparisonChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
	
	public TimeSeriesChart newTimeSeries() {
		TimeSeriesChart g = new TimeSeriesChart(idGenerator.getNext(), "TimeSeriesChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}

	public DetailedTimeSeriesChart newDetailedTimeSeries() {
		DetailedTimeSeriesChart g = new DetailedTimeSeriesChart(idGenerator.getNext(), "DetailedTimeSeriesChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}

	public HistogramChart newHistogram() {
		HistogramChart g = new HistogramChart(idGenerator.getNext(), "HistogramChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
	
	public BoxplotChart newBoxplot() {
		BoxplotChart g = new BoxplotChart(idGenerator.getNext(), "BoxplotChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
	
	public DataTableChart newDataTable() {
		DataTableChart g = new DataTableChart(idGenerator.getNext(), "DataTableChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}

	public HeatmapChart newHeatmap() {
		HeatmapChart g = new HeatmapChart(idGenerator.getNext(), "HeatmapChart", ObjectIdentity.NONE, idGenerator, invocationBuilder.build());
		return g;
	}
}
