package net.pricefx.logic.core;

import java.util.EnumMap;

import static net.pricefx.common.api.PriceFxInterface.FIELD_CUSTOMER_ID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_DCR_TYPE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_ID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_JOBSTATUSTRACKER_JOBNAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_LABEL;
import static net.pricefx.common.api.PriceFxInterface.FIELD_MLTVM_TABLEID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_NAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_PGI_PRICEGRIDID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_PLI_PRICELISTID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_PR_LINEID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_PR_SOURCEID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_RR_SOURCEID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_SKU;
import static net.pricefx.common.api.PriceFxInterface.FIELD_UNIQUENAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_USER_EMAIL;
import static net.pricefx.common.api.PriceFxInterface.FIELD_USER_LAST_NAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_USER_LOGINNAME;

public enum TypeCode {
    NONE(""),

    APPLICATIONPROPERTIES("AP"),
    APIKEY("APIK"),
    BINARYDATA("BD"),
    BUSINESSROLE("BR"),
    BOUNDPARTITION("BPT"),
    CUSTOMER("C", new String[] {FIELD_CUSTOMER_ID}, FIELD_NAME),
    CUSTOMERASSIGNMENT("CA"),
    CUSTOMERATTRIBUTEMETA("CAM"),
    CUSTOMERDESCRIPTION("CDESC"),
    CALCULATIONFLOW("CF"),
    CALCULATEDFIELDSET("CFS"),
    CALCULATIONFLOWTRAIT("CFT"),
    COMMENTHISTORY("CH"),
    CLUSTERNODE("CN"),
    CALCULATIONSCHEDULE("CS"),
    CLAIM("CL", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    CONFIGURATION_WIZARD("CW", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    CONTRACT("CT", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    CONTRACTATTRIBUTEMETA("CTAM"),
    CONTRACTLINEITEM("CTLI"),
    CONTRACTTERMTYPE("CTT", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    CONTRACTTERMTYPEATTRIBUTEMETA("CTTAM"),
    CONTRACTTREE("CTTREE"),
    CUSTOMEREXTENSION("CX", new String[] {FIELD_NAME}),
    CX3("CX3", new String[] {FIELD_NAME}),
    CX6("CX6", new String[] {FIELD_NAME}),
    CX8("CX8", new String[] {FIELD_NAME}),
    CX10("CX10", new String[] {FIELD_NAME}),
    CX20("CX20", new String[] {FIELD_NAME}),
    CX30("CX30", new String[] {FIELD_NAME}),
    CX50("CX50", new String[] {FIELD_NAME}),
    CUSTOMEREXTENSIONATTRIBUTEMETA("CXAM"),
    DATAARCHIVE("DA"),
    DASHBOARD("DB"),
    DATACHANGEREQUEST("DCR", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    DATACHANGEREQUESTITEMAM("DCRAM"),
    DATACHANGEREQUESTITEM("DCRI"),
    DATACHANGEREQUESTLOG("DCRL"),
    DATACHANGEREQUESTMASSCHANGE("DCRMC"),
    DATACHANGEREQUESTTYPE("DCRT"),
    DATAEXPORT("DE"),
    DATAIMPORT("DI"),
    DEALPLANTYPE("DPT"),
    DMDATAMART("DM"),
    DMDATACHUNK("DMDC"),
    DMDATALOAD("DMDL"),
    DMDATASOURCE("DMDS"),
    DMDATAFEED("DMF"),
    DMMODEL("DMM"),
    DMQUERYCACHE("DMQC"),
    DMROLLUP("DMR"),
    DMSIMDATAMART("DMSIM"),
    DMSALESSIMDATAMARTCALCULATION("DMSSCALC"),
    DMVIEW("DMV"),
    DEVICEREGISTRATION("DREG"),
    DWTABLE("DWT"),
    EMAILTASK("ET"),
    EVENTTASK("EVT"),
    PRICINGFORMULA("F"),
    PRICINGFORMULAELEMENT("FE"),
    FORMULAINVARIANT("FI"),
    FORMULAINVARIANTELEMENT("FIE"),
    FORMULAINVARIANTENGINERUN("FIER"),
    FILTERNODE("FN"),
    INLINEDETAILSCONFIGURATION("IDC"),
    INTERNALEVENT("IE"),
    INFORMATIONFLOWPROCESSOR("IFP"),
    INTEGRATIONSTATUSHISTORY("ISH"),
    JOBSTATUSTRACKER("JST", new String[] {FIELD_ID}, FIELD_JOBSTATUSTRACKER_JOBNAME),
    LOGINAUDITTRAIL("LAT"),
    LOOKUPTABLE("LT", new String[] {FIELD_MLTVM_TABLEID}),
    LOOKUPTABLETREE("LTT"),
    LOOKUPTABLEVALUE("LTV"),
    MATRIXLOOKUPTABLEVALUE("MLTV"),
    MATRIXLOOKUPTABLEVALUE2KEY("MLTV2"),
    MATRIXLOOKUPTABLEVALUE3KEY("MLTV3"),
    MATRIXLOOKUPTABLEVALUE4KEY("MLTV4"),
    MATRIXLOOKUPTABLEVALUE5KEY("MLTV5"),
    MATRIXLOOKUPTABLEVALUE6KEY("MLTV6"),
    MATRIXLOOKUPTABLEATTRIBUTEMETA("MLTVM"),
    MANUALPRICELIST("MPL", new String[] {FIELD_ID}, FIELD_LABEL),
    MANUALPRICELISTITEMATTRIBUTEMETA("MPLAM"),
    MANUALPRICELISTITEM("MPLI"),
    MANUALPRICELISTINTEGRATIONTAG("MPLIT"),
    MANUALPRICELISTTREE("MPLT"),
    MODELOBJECT("MO", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    MODELRECORD("MR", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    MODELRECORDATTRIBUTEMETA("MRAM"),
    MODELTYPE("MT", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    PRODUCT("P", new String[] {FIELD_SKU}, FIELD_LABEL),
    PRODUCTATTRIBUTEMETA("PAM"),
    PRICEAPIJOB("PAPIJ"),
    PRODUCTBOMENTRY("PBOME"),
    PRODUCTCOMPETITION("PCOMP"),
    PRODUCTDESCRIPTION("PDESC"),
    PRICEGRID("PG", new String[] {FIELD_ID}, FIELD_LABEL),
    PRICEGRIDITEM("PGI", new String[] {FIELD_PGI_PRICEGRIDID}),
    PRICEGRIDITEMATTRIBUTEMETA("PGIM"),
    PRICEGRIDTREE("PGT"),
    PRICEOPTIMIZATIONMODEL("DMM", new String[] {FIELD_ID}, FIELD_LABEL),
    PARTITIONHELP("PH"),
    PRICELIST("PL", new String[] {FIELD_ID}, FIELD_LABEL),
    PRICELISTITEM("PLI", new String[] {FIELD_PLI_PRICELISTID}),
    PRICELISTITEMATTRIBUTEMETA("PLIM"),
    PRICELISTPRICEGRIDTYPE("PLPGTT"),
    PRICELISTTREE("PLT"),
    PRICERECORD("PR", new String[] {FIELD_RR_SOURCEID, FIELD_PR_LINEID}, FIELD_RR_SOURCEID),
    PRICERECORDATTRIBUTEMETA("PRAM"),
    PREFERENCES("PREF"),
    PARTITION("PT"),
    PASSWORDHISTORY("PWH"),
    PRODUCTEXTENSION("PX", new String[] {FIELD_NAME}),
    PX3("PX3"),
    PX6("PX6"),
    PX8("PX8"),
    PX10("PX10"),
    PX20("PX20"),
    PX30("PX30"),
    PX50("PX50"),
    PRODUCTEXTENSIONATTRIBUTEMETA("PXAM"),
    PRODUCTXREFERENCE("PXREF"),
    PAYOUTRECORD("PYR"),
    PAYOUTRECORDATTRIBUTEMETA("PYRAM"),
    QUOTE("Q", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    QUOTEATTRIBUTEMETA("QAM"),
    QUOTELINEITEM("QLI"),
    QUOTETREE("QT"),
    QUOTETYPE("QTT"),
    QUOTETYPEATTRIBUTEMETA("QTTAM"),
    ROLE("R"),
    REBATEAGREEMENTTEMPLATE("RAT"),
    REBATEAGREEMENTTEMPLATEATTRIBUTEMETA("RATM"),
    REBATEAGREEMENT("RBA", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    REBATEAGREEMENTATTRIBUTEMETA("RBAAM"),
    REBATEAGREEMENTLINEITEM("RBALI"),
    REBATEAGREEMENTTREE("RBAT"),
    REBATETYPE("RBT", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    REBATETYPEATTRIBUTEMETA("RBTAM"),
    REBATERECORD("RR", new String[] {FIELD_UNIQUENAME}, FIELD_LABEL),
    REBATERECORDATTRIBUTEMETA("RRAM"),
    REBATERECORDSET("RRS"),
    REBATERECORDSETCALCULATION("RRSC"),
    REFRESHTOKEN("RT"),
    SECURITYAUDITTRAIL("SAT"),
    SAVEDCHART("SC"),
    SAVEDCHARTTREE("SCT"),
    SIMULATIONITEMATTRIBUTEMETA("SIAM"),
    SIMULATION("SIM"),
    SIMULATIONITEM("SIMI"),
    TFATOKEN("TFA"),
    TODOITEM("TODO"),
    USER("U", new String[] {FIELD_USER_LOGINNAME}, FIELD_USER_LAST_NAME),
    USERGROUP("UG"),
    UPLOADSTATUS("US"),
    WORKFLOW("W"),
    WORKFLOWFORMULA("WF"),
    WORKFLOWFORMULAELEMENT("WFE"),
    MATRIXPRICEGRIDITEM("XPGI"),
    MATRIXPRICELISTITEM("XPLI"),
    MATRIXSIMULATIONITEM("XSIMI");

    public static final EnumMap<TypeCode, TypeCode> METADATA_TYPECODES;

    static {
        METADATA_TYPECODES = new EnumMap(TypeCode.class);
        METADATA_TYPECODES.put(PRODUCT, PRODUCTATTRIBUTEMETA);
        METADATA_TYPECODES.put(PRODUCTEXTENSION, PRODUCTEXTENSIONATTRIBUTEMETA);
        METADATA_TYPECODES.put(CUSTOMER, CUSTOMERATTRIBUTEMETA);
        METADATA_TYPECODES.put(CUSTOMEREXTENSION, CUSTOMEREXTENSIONATTRIBUTEMETA);
        METADATA_TYPECODES.put(CONTRACTTERMTYPE, CONTRACTTERMTYPEATTRIBUTEMETA);
        METADATA_TYPECODES.put(LOOKUPTABLE, MATRIXLOOKUPTABLEATTRIBUTEMETA);
        METADATA_TYPECODES.put(PRICERECORD, PRICERECORDATTRIBUTEMETA);
        METADATA_TYPECODES.put(PRICELISTITEM, PRICELISTITEMATTRIBUTEMETA);
        METADATA_TYPECODES.put(PRICEGRIDITEM, PRICEGRIDITEMATTRIBUTEMETA);
        METADATA_TYPECODES.put(REBATETYPE, REBATETYPEATTRIBUTEMETA);
        METADATA_TYPECODES.put(REBATEAGREEMENTTEMPLATE, REBATEAGREEMENTTEMPLATEATTRIBUTEMETA);
    }

    private final String shortName;
    private final String[] identifierFieldNames;
    private final String labelFieldName;

    TypeCode(String value, String[] identifierFieldNames, String labelFieldName) {
        this.shortName = value;
        this.identifierFieldNames = identifierFieldNames;
        this.labelFieldName = labelFieldName;
    }

    TypeCode(String value, String[] identifierFieldNames) {
        this(value, identifierFieldNames, null);
    }

    TypeCode(String value) {
        this(value, null);
    }

    public String getShortName() {
        return shortName;
    }

    public static TypeCode findByShortName(String shortName) {
        for (TypeCode typeCode : values()) {
            if (typeCode.getShortName().equals(shortName)) {
                return typeCode;
            }
        }
        return null;
    }

    public TypeCode getMetaTypeCode() {
        TypeCode typeCode = METADATA_TYPECODES.get(this);
        if (typeCode == null) {
            throw new IllegalArgumentException("TypeCode.getMetaTypeCode() is not implemented for " + name());
        }
        return typeCode;
    }

    public String[] getIdentifierFieldNames() {
        if (identifierFieldNames == null) {
            throw new IllegalArgumentException("Add implementation in TypeCode.getKeyField() for " + getShortName());
        }
        return identifierFieldNames;
    }

    public String getLabelFieldName() {
        return labelFieldName;
    }

    public String getLabelFieldName2() {
        switch (this) {
            case DATACHANGEREQUEST:
                return FIELD_DCR_TYPE;
            case USER:
                return FIELD_USER_EMAIL;
            default:
                return null;
        }
    }

    public static TypeCode fromClass(Class clazz) {
        try {
            return TypeCode.valueOf(clazz.getSimpleName().toUpperCase());
        } catch (IllegalArgumentException e) {
            //TODO 1. improve the model interfaces so there are only the valid ones 2. be typeCode independent
            return null;
        }
    }
}