package net.pricefx.workflowengine;

import java.util.List;

import net.pricefx.workflowengine.WorkflowInfo.WorkflowStepInfo;

/**
 * Can be executed, undone, redone only once.
 *
 */
public interface WorkflowStep {
	
	String ID = "id";
	String TYPE = "type";
	
	/**
	 * Any action that should be executed before actual denial, approval of step
	 * like sending email.
	 */
	void preExecute(String submitterTypedId);

	/**
	 * Executes step. {@code WorkflowExecutionEngineException} thrown in case
	 * action was not possible to make.
	 * 
	 * @param action
	 * @return true the step is done, false otherwise (multiple approvals pending e.g.)
	 */
	boolean execute(WorkflowStepExecutionAction action, String comment, String selectedApprover, String submitterTypedId);

	String getId();
	WorkflowStepType getType();
	String getPostLogicName();
	void setPostStepLogicFailed(boolean failed);
	boolean getCanAddStepsFromPostStepLogic();

	List<String> getUsers();

	List<String> getPendingUsers();

	/**
	 * Step that is executed immediately without user intervention(Approval, denial...) 
	 * @return
	 */
	public boolean isPassThroughStep();
	
	WorkflowStepInfo getWorkflowStepInfo();

    String getUniqueName();

	boolean writeNamesToWorkflowDefinition();

	public enum WorkflowStepExecutionAction {
		APPROVE, DENY, WITHDRAW
	}
	
	/**
	 * As it sees user.
	 */
	public enum WorkflowStepExecutionStatus {
		INITIALIZED, PREEXECUTED, EXECUTED_DENIED, EXECUTED_APPROVED, EXECUTED_WITHDRAWN, ALREADY_SATISFIED;
		
		public String i18n() {
			throw new UnsupportedOperationException("Unsupported enum " + this);
		}
	}
	
	public enum WorkflowStepType {
		APPROVAL("ApprovalWorkflowStep"),
		WATCHER("WatcherWorkflowStep");
		
		private String id;
		
		private WorkflowStepType(String id) {
			this.id = id;
		}
		
		@Override
		public String toString() {
			return id;
		}		
	}

}
